﻿using System;
using Alachisoft.NCache.Runtime;
using Alachisoft.NCache.Web.Caching;
using Alachisoft.NCache.sample.data;
using System.Threading;
// ===============================================================================
// Alachisoft (R) NCache Sample Code.
// NCache Basic Operations sample
// ===============================================================================
// Copyright © Alachisoft.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
// ===============================================================================


namespace AsynchronousOperations
{
	public class AsynchronousOperations
	{
		public static void Main(string[] args)
		{
			try
			{
				//Initialize cache via 'initializeCache' using 'Cache Name'
				//to be initialized. 
				Cache cache = NCache.InitializeCache("mypartitionedcache");
				cache.Clear();

				//Another method to add item(s) to cache is via CacheItem  object
				Customer customer = new Customer();
				customer.Name = "David Johnes";
				customer.Age = 23;
				customer.Gender = "Male";
				customer.ContactNo = "12345-6789";
				customer.Address = "Silicon Valley, Santa Clara, California";

				DateTime calendar = new DateTime();
				calendar.AddMinutes(1);

                //Adding item asynchronously
                cache.AddAsync("Customer:DavidJohnes", customer, null, string.Empty, string.Empty);
                Thread.Sleep(1000);
                               
                Customer cachedCustomer = (Customer) cache.Get("Customer:DavidJohnes");
                
                printCustomerDetails(cachedCustomer);

                //updating item asynchronously
				customer.Age = 50;
                cache.InsertAsync("Customer:DavidJohnes", customer, null, string.Empty, string.Empty);
                Thread.Sleep(1000);

				//get customer from the cache
				cachedCustomer = (Customer) cache.Get("Customer:DavidJohnes");                
                printCustomerDetails(cachedCustomer);
              

				//Total item count in the cache
				long count = cache.Count;
                Console.WriteLine("Cache Count: " +count);

                //remove the existing customer asynchronously
                cache.RemoveAsync("Customer:DavidJohnes", null, DSWriteOption.None, null);
                Thread.Sleep(1000);

				//try to get the customer again, getting non-existing items returns null
				cachedCustomer = (Customer) cache.Get("Customer:DavidJohnes");
                printCustomerDetails(cachedCustomer);

				//get count again, item count should be 0
				count = cache.Count;
                Console.WriteLine("Cache Count: " + count);

                Console.WriteLine("Cache count after clearing");

                //Clear cache asynchronously
                cache.ClearAsync(null);
                Thread.Sleep(1000);

                Console.WriteLine("Cache Count: " + count);

				//Dispose the cache once done
				cache.Dispose();

				Environment.Exit(0);
			}
			catch (Exception e)
			{
				Console.WriteLine(e.Message);
				Environment.Exit(0);
			}
		}

        /// <summary>
        /// Method for printing detials of customer type.
        /// </summary>
        /// <param name="customer"></param>
		private static void printCustomerDetails(Customer customer)
        {
            if (customer == null) return;
            
            Console.WriteLine();
			Console.WriteLine("Customer Details are as follows: ");
			Console.WriteLine("Name: " + customer.Name);
			Console.WriteLine("Age: " + customer.Age);
			Console.WriteLine("Gender: " + customer.Gender);
			Console.WriteLine("Contact No: " + customer.ContactNo);
			Console.WriteLine("Address: " + customer.Address);
			Console.WriteLine();
		}
	}
}